//-------------------------------------------------------------------------------------------//
// program: LS_calibrate.pde
// author:  Mike Barton
//-------------------------------------------------------------------------------------------//
// try series/shunt permutations to find best values (in perfect_atten[127]) 
// and store results in series/shunt_value[127]
//-------------------------------------------------------------------------------------------//
// Settings               Description
// ========               ===========
// SETTLE_DELAY           number of mS the program waits before reading the value
// MIN_NUMBER_SAMPLES     minimum number of samples to take before checking variance
// MAX_SAMPLE_VARIANCE    the maximum allowed variance between samples to determine a value.
// INTER_SAMPLE_DELAY     number of mS between each sample
//-------------------------------------------------------------------------------------------//

#define SETTLE_DELAY 2000
#define MIN_NUMBER_SAMPLES 3
#define MAX_SAMPLE_VARIANCE 1
#define INTER_SAMPLE_DELAY 500

// configuration 
#define VOL_SDI_PIN 2
#define VOL_SCK_PIN 3
#define VOL_STR_PIN 4

#define ANALOGUE_PINS_OFFSET 14
#define ATTENUATION_PIN 0

#include <avr/interrupt.h>  
#include <avr/io.h>
#include <stdio.h>

byte cmd_pot_shunt    = B00010010;         // command byte to write to pot 0, from datasheet
byte cmd_pot_series   = B00010001;         // command byte to write to pot 1, from datasheet
byte cmd_pot_both     = B00010011;         // command byte to write to pots 0 and 1, from datasheet

// best attenuation values (from ADC read) (ascending volume)
static const int perfect_atten[127] = 
  {1, 2, 3, 4, 5, 6, 7, 9, 10, 11, 
  12, 14, 15, 16, 17, 18, 19, 20, 21, 23, 
  24, 25, 27, 29, 30, 32, 34, 36, 38, 40, 
  43, 45, 47, 49, 51, 53, 55, 57, 59, 62, 
  64, 66, 69, 72, 75, 77, 80, 84, 87, 90, 
  94, 98, 101, 105, 109, 114, 118, 123, 128, 133, 
  138, 143, 149, 155, 161, 167, 173, 180, 187, 193, 
  198, 204, 210, 216, 223, 229, 236, 243, 250, 257, 
  264, 272, 280, 288, 297, 305, 314, 324, 333, 343, 
  353, 363, 374, 384, 396, 407, 419, 431, 444, 457, 
  470, 484, 498, 513, 528, 543, 559, 575, 592, 609, 
  627, 645, 664, 684, 704, 724, 745, 767, 790, 813, 
  836, 861, 886, 912, 938, 966, 994, 
 };
// best attenuation values (from ADC read) (ascending volume)
/*
static const int perfect_atten[127] = 
  {8, 16, 24, 32, 40, 48, 56, 64, 72, 80, 88, 96, 104, 112, 120, 128,
  136, 144, 152, 160, 168, 176, 184, 192, 200, 208, 216, 224, 232, 240, 247, 255,
  264, 272, 279, 287, 295, 303, 311, 319, 327, 335, 343, 351, 359, 367, 375, 383, 
  391, 399, 407, 415, 423, 431, 439, 447, 455, 463, 471, 479, 487, 495, 503, 511, 
  519, 526, 534, 542, 551, 558, 566, 574, 582, 590, 598, 606, 614, 621, 629, 637, 
  645, 654, 662, 670, 677, 685, 693, 701, 709, 718, 726, 734, 742, 750, 757, 765, 
  773, 781, 790, 798, 806, 814, 823, 831, 838, 846, 854, 862, 870, 878, 886, 894, 
  902, 911, 919, 928, 936, 944, 951, 959, 967, 975, 983, 991, 999, 1007, 1015
 };
 */ 

byte series_value[127];    // store what we get
byte shunt_value[127];

int get_ADC_Value(void);

void setup() {
  pinMode(VOL_SDI_PIN, OUTPUT);	
  pinMode(VOL_SCK_PIN, OUTPUT);	
  pinMode(VOL_STR_PIN, OUTPUT);

  pinMode(ATTENUATION_PIN+ANALOGUE_PINS_OFFSET, INPUT);
  
  digitalWrite(VOL_SDI_PIN, LOW);
  digitalWrite(VOL_SCK_PIN, LOW);
  digitalWrite(VOL_STR_PIN, HIGH);
    	
  Serial.begin(9600);
}

void loop() {
  byte i,j,k;
  int variance;
  byte count, remain, reset;
  int value;
  char buffer[31];  
  byte res_series, res_shunt;
  
  for(i=0;i<127;i++) 
  {
    series_value[i]=0;
    shunt_value[i]=0;
  }
  Serial.println("==============================");
  Serial.println("Lightspeed LS_calibrate Run");
  Serial.print("Settle Time=");
  Serial.print(SETTLE_DELAY,DEC);
  Serial.println("mS");
  Serial.print("Sample Count=");
  Serial.print(MIN_NUMBER_SAMPLES,DEC);
  Serial.print(", Inter-Sample Delay=");
  Serial.print(INTER_SAMPLE_DELAY,DEC);
  Serial.println("mS");
  Serial.print("Sample Variance=+/-");
  Serial.println(MAX_SAMPLE_VARIANCE,DEC);
  Serial.println("==============================");   
  Serial.println("Scanning....");
  
  remain=127;
  // loop 1==(remain>0)
  while(remain>0)
  {
    // outer-while-loop increments res_shunt, inner-while-loop decrements res_series
    // res_series is varied 
    res_shunt=1; 
    while(1)
    {
      res_series=255-res_shunt;
      if(res_series>(255-125)) res_series=255;
      else res_series=res_series+125;
      count=0;

      while(1)
      {
        spi_out(cmd_pot_shunt, res_shunt); 
        spi_out(cmd_pot_series, res_series); 

        value=get_ADC_Value();

        if(value>0)    
        {
          for(i=0;i<127;i++)
          {
	    if(value==perfect_atten[i])
            {
              if(series_value[i]==0) 
	      {
                series_value[i]=res_series;
                shunt_value[i]=res_shunt;
                remain--;
              }
              break;
	    }
          }
        }
      
        if(res_series==1) break;
        count++;
        if(count==250) break;
        res_series--;
      } // while res_series
      
      if (remain==0) break;    // if we have them all, exit search loop 
      
      if(res_shunt==255) break;
      res_shunt++;
    
      sprintf(buffer, "res_shunt=%d, remaining=%u" , (res_shunt-1),remain);
      Serial.println(buffer);

    } // while res_shunt
    
    // now re-check values and re-set if necessary...
    reset=0;
    for(i=0;i<127;i++)
    {
      if(series_value[i]>0)
      {
        spi_out(cmd_pot_shunt, shunt_value[i]); 
        spi_out(cmd_pot_series, series_value[i]); 

        value=get_ADC_Value();
        variance=value-perfect_atten[i];
        if(variance<0)variance=0-variance;
        if(variance>1)
        {
          series_value[i]=0;
          shunt_value[i]=0;
          remain++;
          reset++;
        }
      }
    }
    sprintf(buffer, "reset=%u" , reset);
    Serial.println(buffer);
   
  } // while(remain>0)
  
  
  // output results
  Serial.println("Results:");  
  for(i=0;i<127;i++)
  { 
    sprintf(buffer, "%d:[%u/%u], ", i, series_value[i], shunt_value[i]);
    Serial.print(buffer);

    if(i%10==9) Serial.println(" ");
  }
  Serial.println(" ");


  // output results so we can cut'n'paste into our actual program
  Serial.println("Output c arrays:"); 
  Serial.println("static const byte series_value[127] =");
  for(i=0;i<127;i++)
  { 
    Serial.print(series_value[i],DEC);
    if(i<126) Serial.println(", ");
    if(i%10==9) Serial.println(" ");
  }
  Serial.println("};");
  Serial.println("static const byte shunt_value[127] =");
  for(i=0;i<127;i++)
  { 
    Serial.print(shunt_value[i],DEC);
    if(i<126) Serial.println(", ");
    if(i%10==9) Serial.println(" ");
  }
  Serial.println("};");
  
  //re-sample using these settings
  Serial.println("Re-sample Results:");  
  for(i=0;i<127;i++)
  { 
    spi_out(cmd_pot_shunt, shunt_value[i]); 
    spi_out(cmd_pot_series, series_value[i]); 

    value=get_ADC_Value();
    
    Serial.print(value,DEC);
    Serial.print(", ");

    if(i%10==9) Serial.println(" ");
  }
  Serial.println("Completed");
  while(1);
}


int get_ADC_Value(void)
{
  byte i;
  int values[MIN_NUMBER_SAMPLES];
  byte values_count;
  long total_value;
  int value, max_value, min_value, variance;

  delay(SETTLE_DELAY);

  // get intial samples
  values_count=0;
  while(values_count<MIN_NUMBER_SAMPLES)
  {
    values[values_count]=analogRead(ATTENUATION_PIN);
    
    // if this is nowhere near what we are looking for, ignore....
    for(i=0;i<127;i++)
    {
      if(series_value[i]==0)
      {
        variance=values[values_count]-perfect_atten[i];
        if(variance<0)variance=0-variance;
        if(variance<=MAX_SAMPLE_VARIANCE) break;
      }
    }
    if(i==127) return 0;
    
    values_count++;
    delay(INTER_SAMPLE_DELAY);
  }

  // now check for variance & repeat sampling if necessary
  while(1)
  {
    max_value=0; min_value=1024; total_value=0;
    for(i=0;i<MIN_NUMBER_SAMPLES;i++)
    {
      if(values[i]<min_value) min_value=values[i];
      if(values[i]>max_value) max_value=values[i];
      total_value=total_value+values[i];
    }
    if((max_value-min_value)<=MAX_SAMPLE_VARIANCE) break;

    // shift data along
    for(i=1;i<MIN_NUMBER_SAMPLES;i++)
      values[i-1]=values[i];

    values[MIN_NUMBER_SAMPLES-1]=analogRead(ATTENUATION_PIN);
    delay(INTER_SAMPLE_DELAY);
  }
 
  value = total_value/MIN_NUMBER_SAMPLES; 
  if((MAX_SAMPLE_VARIANCE>0) && ((total_value%MIN_NUMBER_SAMPLES)>(MIN_NUMBER_SAMPLES/2))) value++;

  return value;
}


void spi_out(byte cmd, byte data)
{
  byte i;

  // init
  digitalWrite(VOL_SCK_PIN, LOW);
  digitalWrite(VOL_STR_PIN, LOW);

  // output command
  for(i=0;i<8;i++)
  {
    if(cmd & 0x80)
      digitalWrite(VOL_SDI_PIN, HIGH);
    else
      digitalWrite(VOL_SDI_PIN, LOW);

    digitalWrite(VOL_SCK_PIN, HIGH); 
    digitalWrite(VOL_SCK_PIN, LOW);
    cmd = cmd << 1;
  } 
  
  // output value
  for(i=0;i<8;i++)
  {
    if(data & 0x80)
      digitalWrite(VOL_SDI_PIN, HIGH);
    else
      digitalWrite(VOL_SDI_PIN, LOW);

    digitalWrite(VOL_SCK_PIN, HIGH); 
    digitalWrite(VOL_SCK_PIN, LOW);
    data = data << 1;
  } 
  
  digitalWrite(VOL_STR_PIN, HIGH); 
}

